import { applyMiddleware, compose, createStore } from 'redux';
import { connectRouter, ConnectedRouter, routerMiddleware } from 'connected-react-router';
import { connect, Provider } from 'react-redux';
import React from 'react';
import ReactDOM from 'react-dom';
import { createBrowserHistory } from 'history';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

function isType(target, type) {
  return Object.prototype.toString.call(target).toLowerCase() === '[object ' + type + ']';
}

function isFunction(target) {
  return isType(target, 'function');
}

function isString(target) {
  return isType(target, 'string');
}

function isObject(target) {
  return isType(target, 'object');
}

function isArray(target) {
  return isType(target, 'array');
}

function isPromise(v) {
  return v && typeof v.then === 'function';
}

function deepExtend(out) {
  out = out || {};

  for (var i = 1, len = arguments.length; i < len; ++i) {
    var obj = arguments[i];

    if (!obj) {
      continue;
    }

    for (var key in obj) {
      if (!obj.hasOwnProperty(key)) {
        continue;
      }

      if (isObject(obj[key])) {
        out[key] = deepExtend(out[key], obj[key]);
        continue;
      }

      out[key] = obj[key];
    }
  }

  return out;
}

function deepExtendArrayType(out, arr) {
  out = out || {};

  for (var i = 0, len = arr.length; i < len; i++) {
    var obj = arr[i];

    if (!obj) {
      continue;
    }

    for (var key in obj) {
      if (!obj.hasOwnProperty(key)) {
        continue;
      }

      if (isObject(obj[key])) {
        out[key] = deepExtend(out[key], obj[key]);
        continue;
      }

      out[key] = obj[key];
    }
  }

  return out;
}

/**
 * @param {any} obj The object to inspect.
 * @returns {boolean} True if the argument appears to be a plain object.
 */
function isPlainObject(obj) {
  if ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) !== 'object' || obj === null) return false;

  var proto = obj;
  while (Object.getPrototypeOf(proto) !== null) {
    proto = Object.getPrototypeOf(proto);
  }

  return Object.getPrototypeOf(obj) === proto;
}

function combineReducers(reducers) {
  var reducerKeys = Object.keys(reducers);

  return function combination() {
    var preState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    var stateDiff = {};
    var nextState = void 0;
    var hasChanged = false;

    for (var i = 0; i < reducerKeys.length; i++) {
      var reducerKey = reducerKeys[i];

      // execute the booto way reducers
      if (action.type === reducerKey && isBootoActionType(reducerKey)) {
        var _splitActionType = splitActionType(action.type),
            module = _splitActionType.module,
            state = _splitActionType.state;

        var iModule = splitActionType(reducerKey).module;
        var iState = splitActionType(reducerKey).state;
        if (typeof stateDiff[iModule] === 'undefined') {
          stateDiff[iModule] = {};
        }
        if (typeof stateDiff[iModule][iState] === 'undefined') {
          stateDiff[iModule][iState] = {};
        }
        var preStateForKey = preState[module][state];
        var nextStateForKey = stateDiff[iModule][iState] = reducers[reducerKey](preStateForKey, action.payload);
        hasChanged = hasChanged || nextStateForKey !== preStateForKey;
        break;
      }

      // support redux way reducer
      if (!isBootoActionType(reducerKey)) {
        var reducer = reducers[reducerKey];
        var _preStateForKey = preState[reducerKey];
        var _nextStateForKey = reducer(_preStateForKey, action);
        stateDiff[reducerKey] = _nextStateForKey;
        hasChanged = hasChanged || _nextStateForKey !== _preStateForKey;
        break;
      }
    }
    nextState = deepExtend({}, preState, stateDiff);
    return hasChanged ? nextState : preState;
  };
}

/**
 * analyse booto type to {path state method}
 * @param actionType
 * @return {*|string}
 */
function splitActionType(actionType) {
  var typeStrArr = actionType.split('/');
  if (typeStrArr.length !== 3) {
    throw new Error('actionType must be [module]/[type]/[reducer method]');
  }

  var _typeStrArr = slicedToArray(typeStrArr, 3),
      module = _typeStrArr[0],
      state = _typeStrArr[1],
      reducerName = _typeStrArr[2];

  return {
    statePath: module + '/' + state,
    module: module,
    state: state,
    reducerName: reducerName
  };
}

function isBootoActionType(actionType) {
  if (!actionType) return false;
  return actionType.split('/').length === 3;
}

var getMiddleWares = function getMiddleWares(middleWares) {
  if (middleWares && middleWares.length > 0) {
    return applyMiddleware.apply(undefined, toConsumableArray(middleWares));
  }
  return applyMiddleware();
};

var configStore = function configStore(preloadState, reducers, middleWares) {
  var store = createStore(reducers, preloadState, compose(getMiddleWares(middleWares)));
  return store;
};

var promiseMiddle = function promiseMiddle(store) {
  return function (next) {
    return function (action) {
      if (!isPromise(action.payload)) {
        return next(action);
      }

      action.payload.then(function (res) {
        action.payload = res;
        store.dispatch(action);
      }).catch(function (err) {
        action.isError = true;
        action.payload = err;
        store.dispatch(action);
      });
    };
  };
};

function createSingleModuleState(conf) {
  var module = conf.module,
      state = conf.state;

  return defineProperty({}, module, state);
}

function createSingleSRMap(conf) {
  var module = conf.module,
      reducers = conf.reducers;

  var reducerKeys = Object.keys(reducers);
  var stateReducerMap = {};

  // change to type-reducer map
  reducerKeys.forEach(function (key) {
    var fnsObj = reducers[key];
    var fnNames = Object.keys(fnsObj);

    fnNames.forEach(function (fnName) {
      var actionType = module + '/' + key + '/' + fnName;
      stateReducerMap[actionType] = fnsObj[fnName];
    });
  });

  return stateReducerMap;
}

function createMultiModuleState(conf) {
  var mergedState = {};
  for (var i = 0; i < conf.length; i++) {
    var _conf$i = conf[i],
        module = _conf$i.module,
        state = _conf$i.state;

    mergedState[module] = state;
  }
  return mergedState;
}

function createMultiSRMap(conf) {
  var mergedReducerArr = [];
  var mergedReducer = {};
  for (var i = 0; i < conf.length; i++) {
    mergedReducerArr.push(createSingleSRMap(conf[i]));
  }

  deepExtendArrayType(mergedReducer, mergedReducerArr);

  return mergedReducer;
}

function withRouterSR(reducer, history) {
  var routerReducer = {
    router: connectRouter(history)
  };
  return Object.assign(reducer, routerReducer);
}

var bootoDispatch = function bootoDispatch(dispatch) {
  return function (bootoAction) {
    if (!isString(bootoAction) && !isPlainObject(bootoAction)) {
      throw new Error('action must be string | plainObject');
    }

    var action = isString(bootoAction) ? formatStringAction(bootoAction) : formatObjectAction(bootoAction);
    dispatch(action);
  };
};

function formatStringAction(bootoAction) {
  return { type: bootoAction };
}

function formatObjectAction(bootoAction) {
  return bootoAction;
}

function bootoConnect(mapStateToProps, mapDispatchToProps) {

  var mapDispatchToPropsWrapper = void 0;

  if (isFunction(mapDispatchToProps)) {
    mapDispatchToPropsWrapper = function mapDispatchToPropsWrapper(dispatch) {
      var props = mapDispatchToProps();
      props.dispatch = dispatch;
      return props;
    };
  } else if (isObject(mapDispatchToProps)) {
    mapDispatchToPropsWrapper = function mapDispatchToPropsWrapper(dispatch) {
      return Object.assign(mapDispatchToProps, { dispatch: bootoDispatch(dispatch) });
    };
  } else {
    mapDispatchToPropsWrapper = function mapDispatchToPropsWrapper(dispatch) {
      return { dispatch: bootoDispatch(dispatch) };
    };
  }

  return connect(mapStateToProps, mapDispatchToPropsWrapper);
}

var BootoNumber = 0;

function Booto() {

  if (BootoNumber === 1) {
    throw new Error('Booto can new only once!');
  }

  BootoNumber = 1;

  this.history = createBrowserHistory();
  this.middleWares = [routerMiddleware(this.history), promiseMiddle];
}

/**
 * confg
 * {
 *   module: 'key',
 *   state: value,
 *   reducers:
 * }
 * @param conf
 */
Booto.prototype.setup = function (conf) {

  if (!isArray(conf) && !isObject(conf)) {
    throw new Error('conf must be array or object');
  }

  var stateReducerMap = {};

  // single module type
  if (isObject(conf)) {
    this.state = createSingleModuleState(conf);
    stateReducerMap = createSingleSRMap(conf);
  }
  // multiple module type
  else {
      this.state = createMultiModuleState(conf);
      stateReducerMap = createMultiSRMap(conf);
    }

  // reducers combine
  var finalStateReducerMap = withRouterSR(stateReducerMap, this.history);
  this.reducers = combineReducers(finalStateReducerMap);
};

Booto.prototype.use = function (middleWares) {
  if (isArray(middleWares)) {
    this.middleWares = this.middleWares.concat(middleWares);
  } else if (isFunction(middleWares)) {
    this.middleWares.push(middleWares);
  }
};

Booto.prototype.start = function (AppComponent, root) {
  // 创建store
  this.store = configStore(this.state, this.reducers, this.middleWares);

  var rootElement = document.querySelector(root);
  var AppWrapper = React.createElement(
    Provider,
    { store: this.store },
    React.createElement(
      ConnectedRouter,
      { history: this.history },
      AppComponent
    )
  );
  ReactDOM.render(AppWrapper, rootElement);
};

var booto = new Booto();

export default booto;
export { bootoConnect as connect };
//# sourceMappingURL=index.es.js.map
